/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */
import java.io.IOException;

import com.ascert.comms.x25.X25Factory;
import com.ascert.comms.x25.X25Message;
import com.ascert.comms.x25.X25Socket;
import com.ascert.comms.x25.X25SocketFactory;

/**
 *  Test2 - basic send and recv test.
 *
 * Tests sending and receiving normal data, Q-bit data, and M-bit data. 
 *
 *  @version    1.00, 25 April 2003
 *  @author     Rob Walker
 *  @history
 *      25 April 2003    rhw        Created
 */
public class Test2
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    public static X25SocketFactory factory;
    
    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    public static void main (String args[])
            throws Exception
    {
        System.setProperty("com.ascert.comms.driver.farsite.diag",
                "true");
        
        factory = new X25SocketFactory(null);
        
        // create server socket test thread
        MyServer mySrv = new MyServer();
        Thread srvThread = new Thread(mySrv);
        srvThread.start();
        sleepAwhile(2000);
        
        // create client socket test thread
        MyClient myCli = new MyClient();
        Thread cliThread = new Thread(myCli);
        cliThread.start();
    }
    
    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    protected static void sleepAwhile(int period)
    {
        try
        {
            Thread.currentThread().sleep(period);
        }
        catch (InterruptedException ie)
        {
        }
    }
    
    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////


    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    public static class MyServer
            implements Runnable
    {
        X25Socket sock;
        X25Socket srv;
        int bufsz = 200;
        
        public void run()
        {
            // basic server bind and accept
            try
            {
                srv = factory.getServerSocket("23420000222201");
                sock = srv.accept();
                sock.setReceiveBufferSize(bufsz);
                System.out.println("Receive buf set to: " 
                        + sock.getReceiveBufferSize());
                
                X25Message msg = new X25Message(new byte[bufsz]);
                String txt;
                
                while (sock.recv(msg) > 0)
                {
                    txt = new String(msg.getMsg(), 0, msg.getLen());                     
                    System.out.println("Recv: len=" + msg.getLen() 
                            + ",txt=[" + txt + "]" 
                            + ",Q=" + msg.getQbit()
                            + ",M=" + msg.getMbit());
                    //TODO: reset rd len
                    msg.setLen(bufsz);
                }
                
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
            finally
            {
                try
                {
                    if (sock != null)
                    {
                        sock.shutdown();
                    }
                    if (srv != null)
                    {
                        srv.shutdown();
                    }
                }
                catch (IOException ioe)
                {
                }
            }
        }
    }

    
    public static class MyClient
            implements Runnable
    {
        X25Socket sock;
        
        public void run()
        {
            // basic client connect
            try
            {   
                sock = factory.getClientSocket("23420000222201");
                sleepAwhile(5000);
                
                String txt;
                X25Message msg;
                
                // normal data
                for (int ix=0; ix < 20; ix++)
                {
                    txt = "Normal X.25 send: num=" + ix;
                    msg = new X25Message(txt.getBytes());
                    sock.send(msg);
                }
                
                // Q bit data
                for (int ix=0; ix < 20; ix++)
                {
                    txt = "Q-Bit X.25 send: num=" + ix;
                    msg = new X25Message(txt.getBytes(), true, false);
                    sock.send(msg);
                }

                // M bit data - small sends, fit in single receive buffer
                for (int ix=0; ix < 10; ix++)
                {
                    txt = ix + ":0123456789:";
                    msg = new X25Message(txt.getBytes(), false, true);
                    sock.send(msg);
                }
                sleepAwhile(5000);
                
                txt = "last:0123456789:";
                msg = new X25Message(txt.getBytes(), false, false);
                sock.send(msg);
                
                // M bit data - large sends, won't fit in single receive buffer
                for (int ix=0; ix < 30; ix++)
                {
                    txt = ix + ":0123456789:";
                    msg = new X25Message(txt.getBytes(), false, true);
                    sock.send(msg);
                }
                txt = "last:0123456789:";
                msg = new X25Message(txt.getBytes(), false, false);
                sock.send(msg);
                sleepAwhile(5000);

                sock.shutdown();
            }
            catch (IOException ioe)
            {
                System.out.println("Client socket exception: " + ioe);
            }
        }
    }
                
    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////
    
}
