/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */
import java.io.IOException;

import com.ascert.comms.x25.X25Factory;
import com.ascert.comms.x25.X25Socket;
import com.ascert.comms.x25.X25SocketFactory;

/**
 *  Test5 - basic PVC connection test
 *
 * Physical line assignments must be supplied:
 *
 *      java ... Test1 0:0 0:1 
 *                      ^   ^
 *                      |   client PVC line assignment
 *                      server PVC line assignment
 *
 *  @version    1.00, 25 April 2003
 *  @author     Rob Walker
 *  @history
 *      25 April 2003    rhw        Created
 */
public class Test5
{
    
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    public static X25SocketFactory factory;
    
    public static String srvAssign = null;
    public static String cliAssign = null;
    
    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    public static void main (String args[])
            throws Exception
    {
        System.setProperty("com.ascert.comms.driver.farsite.diag",
                "true");
        
        //if enough args supplied, then set server and client socket line
        //assignment information
        if (args.length >= 2)
        {
            srvAssign = args[0];
            cliAssign = args[1];
        }
        else
        {
            System.out.println("Server and client line assignments arguments must be supplied for PVC tests e.g. '0:0 0:2'");
            System.exit(0);
        }
        
        factory = new X25SocketFactory(null);
        
        // create server socket test thread
        MyServer mySrv = new MyServer();
        Thread srvThread = new Thread(mySrv);
        srvThread.start();
        sleepAwhile(2000);
        
        // create client socket test thread
        MyClient myCli = new MyClient();
        Thread cliThread = new Thread(myCli);
        cliThread.start();
        sleepAwhile(20000);
        
        System.out.print("Press a key ...");
        System.in.read();
        myCli.cleanup();
        mySrv.cleanup();    
    }
    
    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    protected static void sleepAwhile(int period)
    {
        try
        {
            Thread.currentThread().sleep(period);
        }
        catch (InterruptedException ie)
        {
        }
    }

    
    protected static void closeAll(X25Socket[] sockArr)
    {
        for (int ix=0; ix < sockArr.length; ix++)
        {
            try
            {   
                if (sockArr[ix] != null)
                {
                    sockArr[ix].shutdown();
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Socket shutdown exception: " + ioe);
            }
        }
    }
    
    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////


    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    // not really a server since this is a PVC test!!!
    public static class MyServer
            implements Runnable
    {
        X25Socket[] socks = new X25Socket[20];
        
        public void run()
        {
            // basic server bind and accept
            try
            {
                for (int ix=0; ix < socks.length; ix++)
                {
                    // force physical adapter/line assignment
                    socks[ix] = factory.getSocket();
                    socks[ix].setPhysicalAssignment(srvAssign);
                    socks[ix].connectPVC("+" + (ix+1));
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
        }

        
        public void cleanup()
        {
            closeAll(socks);
        }
    }

    
    public static class MyClient
            implements Runnable
    {
        X25Socket[] socks = new X25Socket[20];
        
        public void run()
        {
            // basic client connect
            try
            {   
                for (int ix=0; ix < socks.length; ix++)
                {
                    // force physical adapter/line assignment
                    socks[ix] = factory.getSocket();
                    socks[ix].setPhysicalAssignment(cliAssign);
                    socks[ix].connectPVC("+" + (ix+1));
                    sleepAwhile(250);
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Client socket exception: " + ioe);
            }
        }

        
        public void cleanup()
        {
            closeAll(socks);
        }
    }
                
    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////
    
}
