/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */
import java.io.IOException;

import com.ascert.comms.x25.X25Factory;
import com.ascert.comms.x25.X25Socket;
import com.ascert.comms.x25.X25SocketFactory;

/**
 *  Test6 - server side only call user data and raw facilities test
 *
 * Allows a different client X.25 program to connect to this server, optionally
 * including call user data and facilities. The program will dump out what it 
 * sees in the incoming call. 
 *
 *  @version    1.00, 25 April 2003
 *  @author     Rob Walker
 *  @history
 *      25 April 2003    rhw        Created
 */
public class Test6
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    public static X25SocketFactory factory;
    
    //////////////////////////////////////////////////
    // STATIC PUBLIC METHODS
    //////////////////////////////////////////////////

    public static void main (String args[])
            throws Exception
    {
        System.setProperty("com.ascert.comms.driver.farsite.diag",
                "true");
        
        factory = new X25SocketFactory(null);
        
        // create server socket test thread
        MyServer mySrv = new MyServer();
        Thread srvThread = new Thread(mySrv);
        srvThread.start();
        sleepAwhile(20000);
        
        System.out.print("Press a key ...");
        System.in.read();
        mySrv.cleanup();    
    }
    
    //////////////////////////////////////////////////
    // STATIC PROTECTED METHODS
    //////////////////////////////////////////////////

    protected static void sleepAwhile(int period)
    {
        try
        {
            Thread.currentThread().sleep(period);
        }
        catch (InterruptedException ie)
        {
        }
    }

    
    protected static void closeAll(X25Socket[] sockArr)
    {
        for (int ix=0; ix < sockArr.length; ix++)
        {
            try
            {   
                if (sockArr[ix] != null)
                {
                    sockArr[ix].shutdown();
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Socket shutdown exception: " + ioe);
            }
        }
    }
    
    //////////////////////////////////////////////////
    // STATIC PRIVATE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // INSTANCE VARIABLES
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // CONSTRUCTORS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // ACCESSOR METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PUBLIC INSTANCE METHODS
    //////////////////////////////////////////////////


    //////////////////////////////////////////////////
    // INTERFACE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PROTECTED INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // PRIVATE INSTANCE METHODS
    //////////////////////////////////////////////////

    //////////////////////////////////////////////////
    // STATIC INNER CLASSES
    //////////////////////////////////////////////////

    public static class MyServer
            implements Runnable
    {
        X25Socket[] socks = new X25Socket[10];
        X25Socket srv;
        
        public void run()
        {
            // basic server bind and accept
            try
            {
                byte[] cud;
                byte[] facs;
                
                String txt;
                srv = factory.getServerSocket("23420000222201");
                for (int ix=0; ix < socks.length; ix++)
                {
                    socks[ix] = srv.accept();
                    System.out.println("Incoming call: remote=" 
                            + socks[ix].getRemoteAddress() + ",local="
                            + socks[ix].getLocalAddress());
                    
                    // check out call user data comes thru ok
                    cud = socks[ix].getCallUserData();
                    txt = new String(cud, 0, cud.length);
                    System.out.println("Call User Data: len=" + cud.length
                            + ", data=[" + txt + "]");

                    // check out facilities comes thru ok
                    facs = socks[ix].getRawFacilities();
                    System.out.print("Facilities: len=" + facs.length 
                            + ", facs=[");
                    for (int jx = 0; jx < facs.length; jx++)
                    {
                        System.out.print("0x" 
                                + Integer.toHexString(facs[jx]) + ",");
                    }
                    System.out.println("]");
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
        }

        public void cleanup()
        {
            try
            {   
                if (srv != null)
                {
                    srv.shutdown();
                }
            }
            catch (IOException ioe)
            {
                System.out.println("Server socket exception: " + ioe);
            }
            closeAll(socks);
        }
    }

    //////////////////////////////////////////////////
    // NON-STATIC INNER CLASSES
    //////////////////////////////////////////////////
    
}
