/*
 *  Common Public License Notice
 *
 *  The contents of this file are subject to the Common Public License
 *  Version 1.0 (the "License"). You may not use this file except in
 *  compliance with the License. A copy of the License should have been
 *  provided in the release which contained this file. If none was provided,
 *  copies are available at http://www.opensource.org/
 *
 *  Copyright (c) 2003 Ascert LLC.
 *
 *  Contact: Ascert LLC (info@ascert.com)
 *  Contributor(s):
 *  Rob Walker, Ascert LLC
 *
 */
package com.ascert.comms.x25;

import java.io.IOException;

/**
 *  Defines a common factory interface for construction of X25Socket objects. Specific details of how sockets are created will be hidden within the specific drivers which implement this interface.
 *
 *@author    Rob Walker
 *@version   1.00, 21 May 2003
 *@since     1.00
 */
public interface X25Factory
{
    //////////////////////////////////////////////////
    // STATIC VARIABLES
    //////////////////////////////////////////////////

    /** System property containing the name of the default driver implementation class to be used by the {@link X25SocketFactory} */
    public final static String FACTORY_CLASS_PROPERTY =
            "com.ascert.comms.x25.factory.class";

    //////////////////////////////////////////////////
    // SIGNATURES
    //////////////////////////////////////////////////

    /**
     *  Creates a new X25Socket object using this factory. Specific details of what kinds of socket object are returned, and how they are created will be encapsulated within each driver's implementation of this factory. 
     *
     *@return                  a newly created X25Socket object
     *@exception  IOException  if an error occurs creating a new X25Socket
     */
    public X25Socket getSocket()
        throws IOException;

}

