/*
 *                 Common Public License Notice
 * 
 * The contents of this file are subject to the Common Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License should have been 
 * provided in the release which contained this file. If none was provided,
 * copies are available at http://www.opensource.org/
 *
 * Copyright (c) 2003 Ascert LLC.
 *
 * Contact: Ascert LLC (info@ascert.com)
 * Contributor(s):
 *      Rob Walker, Ascert LLC
 *
 */

#include "jni.h"

#include "x25utl.h"

X25_ERRMAP *UTL_lookupmap(X25_ERRMAP *map, int err)
{
    X25_ERRMAP *curr = map;
    
    while (curr->err != 0)
    {
        if (err == curr->err)
        {
            return curr;
        }
        curr++;
    }

    // Not found    
    return NULL;
}


JNIEXPORT void JNICALL
UTL_throw(JNIEnv *env, X25_ERRMAP *map, int err, char *msg, char *exc) 
{
    char excMsg[256];
    X25_ERRMAP *entry = NULL;
    jclass cls;

    // Don't throw if exception pending
    if ((*env)->ExceptionCheck(env) == JNI_TRUE) 
    {
        return;
    }

    // use default message if none provided
    if (msg == NULL) 
    {
        msg = "no further information";
    }

    // check the map (if provided) to decode known errors
    if (map != NULL)
    {
        entry = UTL_lookupmap(map, err);
    }

    //ensure we have an exception if none provided in call
    if (exc == NULL)
    {
        if (entry != NULL && entry->exc != 0)
        {
            // use table entry if found and not 0
            //      (cast removes compile warning on loss of qualifiers) 
            exc = (char *) entry->exc;  
        }
        else
        {
            // default any others to socket exceptions
            exc = "java/net/SocketException"; 
        }
    }

    // format error text, using entry text if any found
    if (entry != NULL && entry->txt != 0)
    {
        sprintf(excMsg, "%s: %s", entry->txt, msg);
    } 
    else 
    {
        sprintf(excMsg, "Unrecognized socket error: %d: %s", err, msg);
    }

    // exception and message set up, so we can now create and throw exception
    cls = (*env)->FindClass(env, exc);
    if (cls != NULL)
    {
        (*env)->ThrowNew(env, cls, excMsg);
    }
    (*env)->DeleteLocalRef(env, cls);
}

