/* WS2FARSYNC.H */

/*

Abstract:

    This header file contains all the definitions for FarSync 
    extensions to WinSock2

    Include this file to enable FarSync X.25
    & OSI features in your application.

--*/


#ifndef _X25SOCK_H_
#define _X25SOCK_H_


#ifndef AF_X25
#define AF_X25 AF_UNKNOWN1
#endif

#ifndef MSG_QUALIFIED
#define MSG_QUALIFIED 0x0008 // NB value chosen to be compatible with another supplier's WS2_OSI.h
#endif


// Socket levels
#define SOL_FARSYNC  0
#define SOL_X25      1
#define SOL_TP       2

// SOL_X25 Socket options
#define X25_QBITINCL  1
#define X25_MBITINCL  2
#define X25_MQHEADER  3

// Bit positions in optional X.25 Data Header
#define XHDR_QBIT 1
#define XHDR_MBIT 2

// Connection state values for FSOPTSTATE
#define FSK_CLOSED      0
#define FSK_CONNECTING  1
#define FSK_CONNECTED   2



// AF_X25 Address family

#define X25_DTE_ADDRLEN 16
typedef struct {
     char          x25_addr[X25_DTE_ADDRLEN];
} x25_address;

struct sockaddr_x25 {
     short         sx25_family;   // AF_X25 
     x25_address   sx25_addr;     // X.25 DTE Address (typically X.121)
};

typedef struct sockaddr_x25 t_sockaddr_x25, * pX25_ADDR;


// AF_CCITT Address Family

struct sockaddr_ccitt {
        short   x25_family;
        u_char  x25hostlen;
        u_char  x25pidlen;
        u_char  x25pid[8];
        u_char  x25_host[16];
        char    x25_ifname[13];
};

typedef struct sockaddr_ccitt t_sockaddr_ccitt, * pCCITT_ADDR;

#define SOCKADDR_CCITT t_sockaddr_ccitt



// Socket Options structures

#pragma pack (push, 1)

typedef struct fsk_port_info {
    int adapter;  /* FarSync X.25 adapter number */
    int line;     /* FarSync X.25 line number */
} t_fsk_port_info, * p_fsk_port_info;

typedef struct x25_causediag {
     unsigned char cause;
     unsigned char diagnostic;
} t_x25_causediag, *p_x25_causediag;

#define X25_CUD_LENGTH 128
typedef struct x25_calluserdata {
     unsigned int  cudlength;
     unsigned char cuddata[X25_CUD_LENGTH];
} t_x25_calluserdata, *p_x25_calluserdata;
#define FSK_MAX_CUD_LENGTH X25_CUD_LENGTH

#define FSK_MAX_FAC_LENGTH 200
typedef struct fsk_raw_facs {
        int length;
        unsigned char data [FSK_MAX_FAC_LENGTH];
} t_fsk_raw_facs, *p_fsk_raw_facs;

#pragma pack (pop)


// SOL_FARSYNC options

#define FSOPTFACS             0  /* t_fsk_raw_facs */
#define FSOPTCUD              1  /* t_x25_calluserdata */
#define FSOPTCAUSE            2  /* t_x25_causediag */
#define FSOPTLINE             3  /* t_fsk_port_info */
#define FSOPTLOCADDR          4  /* struct sockaddr (depending on address family) */
#define FSOPTREMADDR          5  /* struct sockaddr (depending on address family) */
#define FSOPTACCEPTAPPROVAL   6  /* int (BOOLEAN) */
#define FSOPTISSUEACCEPT      7  /* int (value unimportant) */
#define FSOPTSTATE            8  /* int (see above) */
#define FSOPTQUALIFIED        9  /* int (BOOLEAN) */
#define FSOPTSOCKID           10 /* SOCKET */
#define FSOPTCLONELISTEN      11 /* SOCKET */
#define FSOPTSHARENAME        12 /* int (BOOLEAN) */

// SOL_TP options

#define TPOPT_PREFCLASS         0  /* int: 0, 1, 2 or 3 */
#define TPOPT_ALTCLASS          1  /* int: 0, 1, 2 or 3 */
#define TPOPT_IMPLICITFLOW      2  /* int (BOOLEAN) */
#define TPOPT_EXPEDITED         3  /* int (BOOLEAN) */
#define TPOPT_PROTECT           4  /* array of bytes */
#define TPOPT_THROUGHPUT        5  /* array of bytes */
#define TPOPT_TRANSITDELAY      6  /* array of bytes */
#define TPOPT_PRIORITY          7  /* array of bytes */
#define TPOPT_ERROR_RATE        8  /* array of bytes */
#define TPOPT_DISCPARAMS        9  /* array of bytes */
#define TPOPT_TTR               10 /* int (in seconds) */
#define TPOPT_REASON            11 /* int */



// FarSync AF_X25 IOCTL codes

#define SIOCX25SDELAYACPT     _WSAIOW(IOC_VENDOR,FSOPTACCEPTAPPROVAL)
#define SIOCX25SSENDACPT      _WSAIOW(IOC_VENDOR,FSOPTISSUEACCEPT)

#define SIOCX25SCAUSEDIAG     _WSAIOW(IOC_VENDOR,FSOPTCAUSE)
#define SIOCX25SRAWFAC        _WSAIOW(IOC_VENDOR,FSOPTFACS)
#define SIOCX25SCALLUSERDATA  _WSAIOW(IOC_VENDOR,FSOPTCUD)
#define SIOCX25SADAPTER       _WSAIOW(IOC_VENDOR,FSOPTLINE)
#define SIOCX25SLOCALNUA      _WSAIOW(IOC_VENDOR,FSOPTLOCADDR)
#define SIOCX25SREMOTENUA     _WSAIOW(IOC_VENDOR,FSOPTREMADDR)

#define SIOCX25GCAUSEDIAG     _WSAIOR(IOC_VENDOR,FSOPTCAUSE)
#define SIOCX25GRAWFAC        _WSAIOR(IOC_VENDOR,FSOPTFACS)
#define SIOCX25GCALLUSERDATA  _WSAIOR(IOC_VENDOR,FSOPTCUD)
#define SIOCX25GADAPTER       _WSAIOR(IOC_VENDOR,FSOPTLINE)
#define SIOCX25GLOCALNUA      _WSAIOR(IOC_VENDOR,FSOPTLOCADDR)
#define SIOCX25GREMOTENUA     _WSAIOR(IOC_VENDOR,FSOPTREMADDR)

#define SIOCX25GCONSTATE      _WSAIOR(IOC_VENDOR,FSOPTSTATE)

#endif // _X25SOCK_H_


/* Definitions from WS2OSI.H */

#ifndef  _WS2OSI_
#define  _WS2OSI_

/*
 * Protocol values for ISO transport protocols.
 */

#define ISOPROTO_TP_CONS   25           // Transport over CONS
#define ISOPROTO_CLTP_CONS tba          // Connectionless Transport
                                        // over CONS

#define ISOPROTO_TP4_CLNS  29           // Transport class 4 over CLNS
#define ISOPROTO_CLTP_CLNS 30           // Connectionless Transport
                                        // over CLNS

#ifndef _WSHISOTP_						// To avoid conflicts in wshisotp.h
#define ISOPROTO_X25       32           // X.25
#endif

#define ISOPROTO_X25PVC    tba          // Permanent Virtual Circuit
#define ISOPROTO_X25SVC    ISOPROTO_X25 // Switched Virtual Circuit

#define ISOPROTO_TP        ISOPROTO_TP4_CLNS
#define ISOPROTO_CLTP      ISOPROTO_CLTP_CLNS

#define ISOPROTO_TP0_TCP   tba          // Transport class 0 over TCP
                                        // (RFC1006)


// Additional AF_ISO IOCTLs
#define SIO_OSI_X25_GENERATE_RESET   _WSAIOR(IOC_WS2,29)
// Not supported:
//#define SIO_OSI_X25_GET_RESET_DATA   _WSAIOW(IOC_WS2,28)
//#define SIO_OSI_X25_SEND_QUALIFIED   _WSAIO(IOC_WS2,30)
//#define SIO_OSI_X25_GET_QUALIFIED    _WSAIO(IOC_WS2,31)



/*
 *   The maximum size of the transport address (tp_addr field of a
 *   sockaddr_tp structure) is 64. The maximum size of the transport
 *   address (tp_addr field of a sockaddr_ositp structure) is 126.
 */

#ifndef _WSHISOTP_						// To avoid conflicts in wshisotp.h
#define ISO_MAX_ADDR_LENGTH 64
#endif

#define ISO_MAX_EXT_ADDR_LENGTH 126

/*
 *   There are three types of ISO addresses, hierarchical, 
 *   extended-hierarchical and non-hierarchical.  For hierarchical and
 *   extended-hierarchical, the tp_addr field contains both the
 *   transport selector and the NSAP. Extended-hierarchical addresses
 *   also contain a Sub Network address (i.e. X.25 NUA), 
 *   extended addressing for X.25 (used by ISOPROTO_X25 listening sockets 
 *   for Call User Data Routing),
 *   and an optional profile name (not used with FarSync).
 *   For non-hierarchical addresses, tp_addr contains only the transport
 *   address, which must be translated by the ISO TP4 transport provider
 *   into the transport selector and network address.
 */

#ifndef _WSHISOTP_						// To avoid conflicts in wshisotp.h
#define ISO_HIERARCHICAL            0
#define ISO_NON_HIERARCHICAL        1
#endif

#define ISO_EXTENDED_HIERARCHICAL   2

/*
 *   The format of the address structure (sockaddr) to pass to Windows
 *   Sockets APIs.
 *
 */

#ifndef _WSHISOTP_						// To avoid conflicts in wshisotp.h
typedef struct sockaddr_tp {
   u_short tp_family;          // Always AF_ISO
   u_short tp_addr_type;       // ISO_HIERARCHICAL or
                               // ISO_NON_HIERARCHICAL
   u_short tp_taddr_len;       // Length of transport address, <= 52
   u_short tp_tsel_len;        // Length of transport selector, <= 32
                               // 0 if ISO_NON_HIERARCHICAL
   u_char tp_addr[ISO_MAX_ADDR_LENGTH];
} SOCKADDR_TP, *PSOCKADDR_TP, *LPSOCKADDR_TP;

#define ISO_SET_TP_ADDR(sa_tp, port, portlen, node, nodelen)       \
            (sa_tp)->tp_family = AF_ISO;                           \
            (sa_tp)->tp_addr_type = ISO_HIERARCHICAL;              \
            (sa_tp)->tp_tsel_len = (portlen);                      \
            (sa_tp)->tp_taddr_len = (portlen) + (nodelen);         \
            memcpy(&(sa_tp)->tp_addr, (port), (portlen));          \
            memcpy(&(sa_tp)->tp_addr[portlen], (node), (nodelen));
#endif

typedef struct sockaddr_ositp {
   u_short tp_family;          // Always AF_ISO
   u_short tp_addr_type;       // ISO_EXTENDED_HIERARCHICAL
   u_short tp_tsel_len;        // Length of transport selector, <= 32
   u_short tp_nsap_len;        // Length of transport address, <= 20
   u_short tp_snet_len;        // Sub Network address length, <= 18
   u_short tp_ext_addr_len;    // Extended address length, <= 40
   u_short tp_optname_len;     // Option profile name length, <= 16
   u_char  tp_addr[ISO_MAX_EXT_ADDR_LENGTH];                                                   
} SOCKADDR_OSITP, *PSOCKADDR_OSITP, *LPSOCKADDR_OSITP;

#define ISO_SET_OSITP_ADDR(sa_tp, tsel, tsellen, nsap, nsaplen,    \
                           snpa, snpalen, extaddr, extaddrlen,     \
                           optname, optnamelen) {                  \
            u_char * paddr;                                        \
            (sa_tp)->tp_family = AF_ISO;                           \
            (sa_tp)->tp_addr_type = ISO_EXTENDED_HIERARCHICAL;     \
            (sa_tp)->tp_tsel_len = (tsellen);                      \
            (sa_tp)->tp_nsap_len = (nsaplen);                      \
            (sa_tp)->tp_snet_len = (snpalen);                      \
            (sa_tp)->tp_ext_addr_len = (extaddrlen);               \
            (sa_tp)->tp_optname_len = (optnamelen);                \
            memcpy(&(sa_tp)->tp_addr, (tsel), (tsellen));          \
            paddr = &(sa_tp)->tp_addr[tsellen];                    \
            memcpy(paddr, (nsap), (nsaplen));                      \
            paddr += (nsaplen);                                    \
            memcpy(paddr, (snpa), (snpalen));                      \
            paddr += (snpalen);                                    \
            memcpy(paddr, (extaddr), (extaddrlen));                \
            paddr += (extaddrlen);                                 \
            memcpy(paddr, (optname), (optnamelen));                \
        }

#define ISO_OSITP_ADDR_LEN(sa_tp)                                  \
            (sa_tp)->tp_tsel_len +                                 \
            (sa_tp)->tp_nsap_len +                                 \
            (sa_tp)->tp_snet_len +                                 \
            (sa_tp)->tp_ext_addr_len +                             \
            (sa_tp)->tp_optname_len




typedef struct _osispecflowparam {
	u_short paramID;			// Parameter ID.
	u_short paramLength;		// Length of following data.
	u_char  param;				// Parameter data.
} OSISPECFLOWPARAM;

// This parameter specifies the ISO 8208/X25 call facilities to be used.
#define OSI_PARAM_ID_X25_CALL_FACILITY	99

#endif // _WS2OSI_

